*********************************************************************
* explego.asm                                                       *
*                                                                   *
* Handy Board Expansion Board standard libraries                    *
*                                                                   *
* Fred Martin        fredm@media.mit.edu         3 December 1998    *
*********************************************************************

* 6811 registers
BASE		equ	$1000	; register base
PORTA		equ	$1000	; Port A
ADCTL		equ	$1030	; A/D Control/status Register
ADR1		equ	$1031	; A/D Result Register 1
TOC4INT		equ	$e2	; Timer Output Compare 4

* zero page definitions
SYSTIME		equ	$15	; lowest byte of system time
SYSTIMEHI	equ	$14	; get the word here

* latch address and storage
EXPLATCH	equ	$4000	; HW address of servo, analog latch
EXPVAL		equ	$31	; RAM address for exp latch value
LEGOBIT		equ	%00010000 ; bit mask for digital 8 out

	org	MAIN_START

* tick and velocity counts
variable_lego0_counts:		FDB	0
variable_lego0_velocity:	FDB	0
variable_lego0:			FDB	0
lego0_last:			FDB	0
lego0_state:			FCB	0

variable_lego1_counts:		FDB	0
variable_lego1_velocity:	FDB	0
variable_lego1:			FDB	0
lego1_last:			FDB	0
lego1_state:			FCB	0

variable_lego2_counts:		FDB	0
variable_lego2_velocity:	FDB	0
variable_lego2:			FDB	0
lego2_last:			FDB	0
lego2_state:			FCB	0

variable_lego3_counts:		FDB	0
variable_lego3_velocity:	FDB	0
variable_lego3:			FDB	0
lego3_last:			FDB	0
lego3_state:			FCB	0


* install module into 1 kHz IC system interrupt on TOC4
subroutine_initialize_module:
	ldx	#$BF00			; pointer to interrupt base
* install ourselves onto system interrupt
* get current vector; poke such that when we finish, we go there 
	ldd	TOC4INT,X		; SystemInt on TOC4
	std	interrupt_code_exit+1

* install ourself as new vector
	ldd	#interrupt_code_start
	std	TOC4INT,X

* reset encoder variables
	ldx	#variable_lego0_counts
	clra
clrloop	staa	0,x
	inx
	cpx	#lego3_state+1
	bne	clrloop

	rts

* phase 0: turn on 9v supply
* phase 1: turn off 9v supply; calc velocities
* phase 2: take analog readings
* phase 3: calc positions
interrupt_code_start:
	ldaa	SYSTIME
	anda	#3		* calc state at 250 Hz
	beq	phase0
	cmpa	#1
	beq	phase1
	cmpa	#2
	bne	phase3p
	jmp	phase2
phase3p	cmpa	#3
	bne	local_exit
	jmp	phase3
phase0:
* turn on 9v supply
	ldaa	EXPVAL
	oraa	#LEGOBIT
	staa	EXPVAL
	staa	EXPLATCH
local_exit jmp	interrupt_code_exit
	
phase1:
* turn off 9v supply
	ldaa	EXPVAL
	anda	#$ff-LEGOBIT
	staa	EXPVAL
	staa	EXPLATCH

* calc velocities?  update at 1.024 hz
	ldaa	SYSTIMEHI
	anda	#3
	bne	vel99
	ldaa	SYSTIME
	cmpa	#1		* we're in phase 1 here!
	bne	vel99
	
	ldd	variable_lego0_counts
	subd	lego0_last
	std	variable_lego0_velocity
	ldd	variable_lego0_counts
	std	lego0_last
	
	ldd	variable_lego1_counts
	subd	lego1_last
	std	variable_lego1_velocity
	ldd	variable_lego1_counts
	std	lego1_last
	
	ldd	variable_lego2_counts
	subd	lego2_last
	std	variable_lego2_velocity
	ldd	variable_lego2_counts
	std	lego2_last
	
	ldd	variable_lego3_counts
	subd	lego3_last
	std	variable_lego3_velocity
	ldd	variable_lego3_counts
	std	lego3_last
	
vel99	jmp	interrupt_code_exit
	
exp_analog:
	pshb
	bsr	setmux		; switch analog input to A/D
	pulb
analog_read stab	ADCTL		; begin analog conversion
* wait 32 cycles for analog reading to happen
	ldaa	#6		; 2
waitlp	deca			; 2
	bne	waitlp		; 3
	ldab	ADR1		; get analog read
	rts			; b has reading, a has 0

* A contains number to write to latch
* analog muxes are hooked up to high 3 bits
setmux
	ldab	EXPVAL
	andb	#%00011111	; mask off mux bits
	aba			; add em back
	staa	EXPLATCH	; and store
	staa	EXPVAL	
	rts

phase2:
* take analog reads
* lego sensors are mux pos'ns 4,5,6,7 on analog 1
* corresponding to $80, $A0, $c0, $E0 on upper byte of mux routine
	ldd	#$8001
	bsr	exp_analog
	stab	variable_lego0+1

	ldd	#$a001
	bsr	exp_analog
	stab	variable_lego1+1

	ldd	#$c001
	bsr	exp_analog
	stab	variable_lego2+1

	ldd	#$e001
	bsr	exp_analog
	stab	variable_lego3+1
	bra	interrupt_code_exit

phase3:	
	ldab	variable_lego0+1
	bsr	munge_raw_value		* B has 0, 1, 2, or 3
	ldaa	lego0_state
	lsla
	lsla				* shift up for table
	stab	lego0_state		* save for next time
	bsr	table_lookup
	ldd	variable_lego0_counts
	addd	0,x
	std	variable_lego0_counts
	
	ldab	variable_lego1+1
	bsr	munge_raw_value		* B has 0, 1, 2, or 3
	ldaa	lego1_state
	lsla
	lsla				* shift up for table
	stab	lego1_state		* save for next time
	bsr	table_lookup
	ldd	variable_lego1_counts
	addd	0,x
	std	variable_lego1_counts
	
	ldab	variable_lego2+1
	bsr	munge_raw_value		* B has 0, 1, 2, or 3
	ldaa	lego2_state
	lsla
	lsla				* shift up for table
	stab	lego2_state		* save for next time
	bsr	table_lookup
	ldd	variable_lego2_counts
	addd	0,x
	std	variable_lego2_counts
	
	ldab	variable_lego3+1
	bsr	munge_raw_value		* B has 0, 1, 2, or 3
	ldaa	lego3_state
	lsla
	lsla				* shift up for table
	stab	lego3_state		* save for next time
	bsr	table_lookup
	ldd	variable_lego3_counts
	addd	0,x
	std	variable_lego3_counts
* fall into exit

interrupt_code_exit:
	jmp	$0000		/* this value poked in by init routine */

munge_raw_value
* input raw reading in B, output 0, 1, 2, or 3 in B
	subb	#48
	andb	#$C0
	lsrb
	lsrb
	lsrb
	lsrb
	lsrb
	lsrb
	rts	

* A has mux#, B has A/D port#
table_lookup
	aba	
	tab
	lslb				* b has offset into table
	ldx	#counter_table
	abx
	rts

counter_table
* $0 is undefined transition
        fdb 0
	fdb 1
	fdb -1
	fdb $0

        fdb -1
	fdb 0
	fdb $0
	fdb 1

        fdb 1
	fdb $0
	fdb 0
	fdb -1

        fdb $0
	fdb -1
	fdb 1
	fdb 0

