*********************************************************************
* libexpbd.asm                                                      *
*                                                                   *
* Handy Board Expansion Board standard libraries                    *
*                                                                   *
* Fred Martin        fredm@media.mit.edu           03 March 1999    *
*********************************************************************

* 6811 registers
ADCTL		equ	$1030	; A/D Control/status Register
ADR1		equ	$1031	; A/D Result Register 1

* latch address and storage
EXPLATCH	equ	$4000	; HW address of servo, analog latch
USERLATCH	equ	$5000	; user output latch
EXPVAL		equ	$31	; RAM address for exp latch value
USERVAL		equ	$32	; storage for user latch output
DIG8BIT		equ	%00001000 ; bit mask for digital 8 out

	org	MAIN_START

subroutine_initialize_module:
* initialize digital outs to all zero
	clra
	staa	USERVAL
	staa	USERLATCH
* initialize latch to zero
	staa	EXPVAL
	staa	EXPLATCH
	rts

* A has mux#, B has A/D port#
subroutine__exp_analog:
	andb	#7		; keep in range
	pshb
	sei			; begin protected access to A/D
	bsr	setmux		; switch analog input to A/D
	pulb
	stab	ADCTL		; begin analog conversion
* wait 32 cycles for analog reading to happen
	ldaa	#6		; 2
waitlp	deca			; 2
	bne	waitlp		; 3
	ldab	ADR1		; get analog read
	cli			; end protected access to A/D
	rts			; b has reading, a has 0

* A contains number to write to latch
* analog muxes are hooked up to high 3 bits
setmux
	ldab	EXPVAL
	andb	#%00011111	; mask off mux bits
	asla
	asla
	asla
	asla
	asla			; move into position
	aba			; add em back
	staa	EXPLATCH	; and store
	staa	EXPVAL	
	rts

subroutine_set_digital_out:
	tba
	anda	#%11111000
	bne	set_dig8
	bsr	set_or_clear
	sei			; protected access to latch state
	oraa	USERVAL		; or bit in A with latch state
setulatch staa	USERLATCH
	staa	USERVAL
	cli
	rts

set_dig8
	sei
	ldaa	EXPVAL
	oraa	#DIG8BIT
setelatch staa	EXPVAL
	staa	EXPLATCH
	cli
	rts

clear_dig8
	sei
	ldaa	EXPVAL
	anda	#$ff-DIG8BIT
	bra	setelatch
	
test_dig8
	clrb
	ldaa	EXPVAL	
	anda	#DIG8BIT
	bne	return_true
	rts
		
subroutine_clear_digital_out:
	tba
	anda	#%11111000
	bne	clear_dig8
	bsr	set_or_clear
	sei			; protected access to latch state
	coma			; all bits but selected go to 1
	anda	USERVAL		; mask selected to 0
	bra	setulatch

subroutine_test_digital_out:
	tba
	anda	#%11111000
	bne	test_dig8
	bsr	set_or_clear	; A has bit in position to be tested
	clrb
	anda	USERVAL
	beq	return_false
return_true	
	clra
	incb
return_false			
	rts

set_or_clear
	andb	#7		; keep in range 0 to 7
	clra
	sec			; will be rotating carry into A
sdo20	rola
	decb
	bpl	sdo20		; exit loop when count negative
	rts


